@@ -22,25 +22,22 @@
 #include <linux/netfilter_ipv4/ipt_set.h>
 #include "libipt_set.h"
 
-/* Function which prints out usage message. */
 static void SET_help(void)
 {
-	printf("SET v%s options:\n"
+	printf("SET target options:\n"
 	       " --add-set name flags\n"
 	       " --del-set name flags\n"
 	       "		add/del src/dst IP/port from/to named sets,\n"
 	       "		where flags are the comma separated list of\n"
-	       "		'src' and 'dst'.\n"
-	       "\n", IPTABLES_VERSION);
+	       "		'src' and 'dst'.\n");
 }
 
 static const struct option SET_opts[] = {
-	{"add-set",   1, 0, '1'},
-	{"del-set",   1, 0, '2'},
-	{0}
+	{"add-set",   1, NULL, '1'},
+	{"del-set",   1, NULL, '2'},
+	{ }
 };
 
-/* Initialize the target. */
 static void SET_init(struct xt_entry_target *target)
 {
 	struct ipt_set_info_target *info =
@@ -81,8 +78,6 @@ parse_target(char **argv, int invert, unsigned int *flags,
 	*flags = 1;
 }
 
-/* Function which parses command options; returns true if it
-   ate an option */
 static int SET_parse(int c, char **argv, int invert, unsigned int *flags,
                      const void *entry, struct xt_entry_target **target)
 {
@@ -105,7 +100,6 @@ static int SET_parse(int c, char **argv, int invert, unsigned int *flags,
 	return 1;
 }
 
-/* Final check; must specify at least one. */
 static void SET_check(unsigned int flags)
 {
 	if (!flags)
@@ -133,7 +127,6 @@ print_target(const char *prefix, const struct ipt_set_info *info)
 	printf(" ");
 }
 
-/* Prints out the targinfo. */
 static void SET_print(const void *ip, const struct xt_entry_target *target,
                       int numeric)
 {
@@ -144,7 +137,6 @@ static void SET_print(const void *ip, const struct xt_entry_target *target,
 	print_target("del-set", &info->del_set);
 }
 
-/* Saves the union ipt_targinfo in parsable form to stdout. */
 static void SET_save(const void *ip, const struct xt_entry_target *target)
 {
 	struct ipt_set_info_target *info =
@@ -154,11 +146,12 @@ static void SET_save(const void *ip, const struct xt_entry_target *target)
 	print_target("--del-set", &info->del_set);
 }
 
-static struct iptables_target set_target = {
+static struct xtables_target set_tg_reg = {
 	.name		= "SET",
-	.version	= IPTABLES_VERSION,
-	.size		= IPT_ALIGN(sizeof(struct ipt_set_info_target)),
-	.userspacesize	= IPT_ALIGN(sizeof(struct ipt_set_info_target)),
+	.version	= XTABLES_VERSION,
+	.family		= PF_INET,
+	.size		= XT_ALIGN(sizeof(struct ipt_set_info_target)),
+	.userspacesize	= XT_ALIGN(sizeof(struct ipt_set_info_target)),
 	.help		= SET_help,
 	.init		= SET_init,
 	.parse		= SET_parse,
@@ -170,5 +163,5 @@ static struct iptables_target set_target = {
 
 void _init(void)
 {
-	register_target(&set_target);
+	xtables_register_target(&set_tg_reg);
 }
