@@ -10,20 +10,17 @@
 #include <linux/types.h>
 #include <linux/pkt_sched.h>
 
-/* Function which prints out usage message. */
 static void
 CLASSIFY_help(void)
 {
 	printf(
-"CLASSIFY target v%s options:\n"
-"  --set-class [MAJOR:MINOR]    Set skb->priority value\n"
-"\n",
-IPTABLES_VERSION);
+"CLASSIFY target options:\n"
+"  --set-class [MAJOR:MINOR]    Set skb->priority value\n");
 }
 
 static const struct option CLASSIFY_opts[] = {
 	{ "set-class", 1, NULL, '1' },
-	{ }
+	{ .name = NULL }
 };
 
 static int CLASSIFY_string_to_priority(const char *s, unsigned int *p)
@@ -37,8 +34,6 @@ static int CLASSIFY_string_to_priority(const char *s, unsigned int *p)
 	return 0;
 }
 
-/* Function which parses command options; returns true if it
-   ate an option */
 static int
 CLASSIFY_parse(int c, char **argv, int invert, unsigned int *flags,
       const void *entry,
@@ -79,7 +74,6 @@ CLASSIFY_print_class(unsigned int priority, int numeric)
 	printf("%x:%x ", TC_H_MAJ(priority)>>16, TC_H_MIN(priority));
 }
 
-/* Prints out the targinfo. */
 static void
 CLASSIFY_print(const void *ip,
       const struct xt_entry_target *target,
@@ -91,7 +85,6 @@ CLASSIFY_print(const void *ip,
 	CLASSIFY_print_class(clinfo->priority, numeric);
 }
 
-/* Saves the union ipt_targinfo in parsable form to stdout. */
 static void
 CLASSIFY_save(const void *ip, const struct xt_entry_target *target)
 {
@@ -103,23 +96,9 @@ CLASSIFY_save(const void *ip, const struct xt_entry_target *target)
 }
 
 static struct xtables_target classify_target = { 
-	.family		= AF_INET,
+	.family		= AF_UNSPEC,
 	.name		= "CLASSIFY",
-	.version	= IPTABLES_VERSION,
-	.size		= XT_ALIGN(sizeof(struct xt_classify_target_info)),
-	.userspacesize	= XT_ALIGN(sizeof(struct xt_classify_target_info)),
-	.help		= CLASSIFY_help,
-	.parse		= CLASSIFY_parse,
-	.final_check	= CLASSIFY_final_check,
-	.print		= CLASSIFY_print,
-	.save		= CLASSIFY_save,
-	.extra_opts	= CLASSIFY_opts,
-};
-
-static struct xtables_target classify_target6 = { 
-	.family		= AF_INET6,
-	.name		= "CLASSIFY",
-	.version	= IPTABLES_VERSION,
+	.version	= XTABLES_VERSION,
 	.size		= XT_ALIGN(sizeof(struct xt_classify_target_info)),
 	.userspacesize	= XT_ALIGN(sizeof(struct xt_classify_target_info)),
 	.help		= CLASSIFY_help,
@@ -133,5 +112,4 @@ static struct xtables_target classify_target6 = {
 void _init(void)
 {
 	xtables_register_target(&classify_target);
-	xtables_register_target(&classify_target6);
 }
