@@ -1,7 +1,7 @@
 /* Shared library add-on to iptables to add limit support.
  *
- * Jrme de Vivie   <devivie@info.enserb.u-bordeaux.fr>
- * Herv Eychenne    <rv@wallfire.org>
+ * Jérôme de Vivie   <devivie@info.enserb.u-bordeaux.fr>
+ * Hervé Eychenne    <rv@wallfire.org>
  */
 
 #include <stdio.h>
@@ -12,27 +12,26 @@
 #include <stddef.h>
 #include <linux/netfilter/x_tables.h>
 /* For 64bit kernel / 32bit userspace */
-#include "../include/linux/netfilter/xt_limit.h"
+#include <linux/netfilter/xt_limit.h>
 
 #define XT_LIMIT_AVG	"3/hour"
 #define XT_LIMIT_BURST	5
 
-/* Function which prints out usage message. */
 static void limit_help(void)
 {
 	printf(
-"limit v%s options:\n"
+"limit match options:\n"
 "--limit avg			max average match rate: default "XT_LIMIT_AVG"\n"
 "                                [Packets per second unless followed by \n"
 "                                /sec /minute /hour /day postfixes]\n"
-"--limit-burst number		number to match in a burst, default %u\n"
-"\n", IPTABLES_VERSION, XT_LIMIT_BURST);
+"--limit-burst number		number to match in a burst, default %u\n",
+XT_LIMIT_BURST);
 }
 
 static const struct option limit_opts[] = {
 	{ "limit", 1, NULL, '%' },
 	{ "limit-burst", 1, NULL, '$' },
-	{ }
+	{ .name = NULL }
 };
 
 static
@@ -71,7 +70,6 @@ int parse_rate(const char *rate, u_int32_t *val)
 	return 1;
 }
 
-/* Initialize the match. */
 static void limit_init(struct xt_entry_match *m)
 {
 	struct xt_rateinfo *r = (struct xt_rateinfo *)m->data;
@@ -87,8 +85,6 @@ static void limit_init(struct xt_entry_match *m)
 			   "Sorry: burst too large for that avg rate.\n");
 */
 
-/* Function which parses command options; returns true if it
-   ate an option */
 static int
 limit_parse(int c, char **argv, int invert, unsigned int *flags,
             const void *entry, struct xt_entry_match **match)
@@ -145,7 +141,6 @@ static void print_rate(u_int32_t period)
 	printf("%u/%s ", rates[i-1].mult / period, rates[i-1].name);
 }
 
-/* Prints out the matchinfo. */
 static void
 limit_print(const void *ip, const struct xt_entry_match *match, int numeric)
 {
@@ -154,7 +149,6 @@ limit_print(const void *ip, const struct xt_entry_match *match, int numeric)
 	printf("burst %u ", r->burst);
 }
 
-/* FIXME: Make minimalist: only print rate if not default --RR */
 static void limit_save(const void *ip, const struct xt_entry_match *match)
 {
 	struct xt_rateinfo *r = (struct xt_rateinfo *)match->data;
@@ -165,23 +159,9 @@ static void limit_save(const void *ip, const struct xt_entry_match *match)
 }
 
 static struct xtables_match limit_match = {
-	.family		= AF_INET,
+	.family		= AF_UNSPEC,
 	.name		= "limit",
-	.version	= IPTABLES_VERSION,
-	.size		= XT_ALIGN(sizeof(struct xt_rateinfo)),
-	.userspacesize	= offsetof(struct xt_rateinfo, prev),
-	.help		= limit_help,
-	.init		= limit_init,
-	.parse		= limit_parse,
-	.print		= limit_print,
-	.save		= limit_save,
-	.extra_opts	= limit_opts,
-};
-
-static struct xtables_match limit_match6 = {
-	.family		= AF_INET6,
-	.name		= "limit",
-	.version	= IPTABLES_VERSION,
+	.version	= XTABLES_VERSION,
 	.size		= XT_ALIGN(sizeof(struct xt_rateinfo)),
 	.userspacesize	= offsetof(struct xt_rateinfo, prev),
 	.help		= limit_help,
@@ -195,5 +175,4 @@ static struct xtables_match limit_match6 = {
 void _init(void)
 {
 	xtables_register_match(&limit_match);
-	xtables_register_match(&limit_match6);
 }
